@@ -39,6 +39,7 @@
 
 #ifdef GOFFICE_WITH_GTK
 #include <goffice/gtk/goffice-gtk.h>
+#include <gtk/gtkcombobox.h>
 #include <gtk/gtkspinbutton.h>
 #include <gtk/gtktogglebutton.h>
 #endif
@@ -117,18 +118,18 @@ calc_circle_parameters (GogViewAllocation const *area, GogChartMapPolarData *dat
 		x_max = y_max = 1.0;
 	} else {
 		double x;
-		if (data->th0 > 2.0 * M_PI) {
+
+		if (data->th0 < -2.0 * M_PI) {
 			x = data->th0 - fmod (data->th0, 2.0 * M_PI);
 			data->th0 -= x;
 			data->th1 -= x;
-		} else if (data->th1 < - 2.0 * M_PI) {
+		} else if (data->th1 > 2.0 * M_PI) {
 			x = data->th1 - fmod (data->th1, 2.0 * M_PI);
-			data->th0 -= x;
-			data->th1 -= x;
+			data->th0 += x;
+			data->th1 += x;
 		}
 		if (data->th1 - data->th0 > go_add_epsilon (2 * M_PI)) 
-			data->th1 = data->th0 + 
-				fmod (data->th1 - data->th0, 2.0 * M_PI);
+			data->th1 = data->th0 + 2 * M_PI; 
 
 		x_min = x_max = y_min = y_max = 0;
 		x = cos (-data->th0);
@@ -142,11 +143,14 @@ calc_circle_parameters (GogViewAllocation const *area, GogChartMapPolarData *dat
 
 		if (0 > data->th0 && 0 < data->th1)
 			x_max = 1.0;
-		if (M_PI / 2.0 > data->th0 && M_PI / 2.0 < data->th1)
+		if ((       M_PI / 2.0 > data->th0 &&        M_PI / 2.0 < data->th1) ||
+		    (-3.0 * M_PI / 2.0 > data->th0 && -3.0 * M_PI / 2.0 < data->th1))
 			y_min = -1.0;
-		if (M_PI > data->th0 && M_PI < data->th1)
+		if (( M_PI > data->th0 &&  M_PI < data->th1) ||
+		    (-M_PI > data->th0 && -M_PI < data->th1))
 			x_min = -1.0;
-		if (3.0 * M_PI / 2.0 > data->th0 && 3.0 * M_PI / 2.0 < data->th1)
+		if (( 3.0 * M_PI / 2.0 > data->th0 &&  3.0 * M_PI / 2.0 < data->th1) ||
+		    (-      M_PI / 2.0 > data->th0 && -      M_PI / 2.0 < data->th1))
 			y_max = 1.0;
 	}
 	data->rx = area->w / (x_max - x_min);
@@ -284,9 +288,11 @@ gog_chart_map_new (GogChart *chart, GogViewAllocation const *area,
 			}
 		case GOG_AXIS_SET_RADAR:
 			{
-				double minimum, maximum;
 				GogChartMapPolarData *data = g_new (GogChartMapPolarData, 1);
-				
+				double minimum, maximum;
+				double z_rotation = gog_axis_get_circular_rotation (axis0) * M_PI / 180.0;
+				double perimeter;
+	
 				map->axis_map[0] = gog_axis_map_new (axis0, 0.0, 1.0);
 				gog_axis_map_get_bounds (map->axis_map[0], &minimum, &maximum);
 				if (gog_axis_is_discrete (axis0)) {
@@ -295,11 +301,12 @@ gog_chart_map_new (GogChart *chart, GogViewAllocation const *area,
 					calc_polygon_parameters (area, data, fill_area);
 					gog_axis_map_free (map->axis_map[0]);
 					map->axis_map[0] = gog_axis_map_new (axis0, 
-						- M_PI / 2.0,
+						- M_PI / 2.0 + z_rotation,
 						2.0 * M_PI * (maximum - minimum) / (maximum - minimum + 1));
 				} else {
-					minimum *= 2.0 * M_PI / 360.0;
-					maximum *= 2.0 * M_PI / 360.0;
+					perimeter = gog_axis_get_polar_perimeter (axis0);
+					minimum = minimum * 2.0 * M_PI / perimeter + z_rotation;
+					maximum = maximum * 2.0 * M_PI / perimeter + z_rotation;
 					data->th0 = minimum;
 					data->th1 = maximum;
 					calc_circle_parameters (area, data, fill_area);
@@ -344,7 +351,7 @@ gog_chart_map_new (GogChart *chart, GogViewAllocation const *area,
 void
 gog_chart_map_2D_to_view (GogChartMap *map, double x, double y, double *u, double *v)
 {
-	return (map->map_2D_to_view) (map, x, y, u, v);
+	(map->map_2D_to_view) (map, x, y, u, v);
 }
 
 /**
@@ -415,7 +422,7 @@ enum {
 	CHART_PROP_0,
 	CHART_PROP_CARDINALITY_VALID,
 	CHART_PROP_PLOT_AREA,
-	CHART_PROP_PLOT_AREA_IS_MANUAL
+	CHART_PROP_PLOT_AREA_IS_MANUAL,
 };
 
 static GType gog_chart_view_get_type (void);
@@ -517,7 +524,7 @@ gog_chart_populate_editor (GogObject *gobj,
 			   GOCmdContext *cc)
 {
 	static guint chart_pref_page = 0;
-	
+
 	(GOG_OBJECT_CLASS(chart_parent_klass)->populate_editor) (gobj, editor, dalloc, cc);
 
 	gog_editor_set_store_page (editor, &chart_pref_page);
@@ -825,7 +832,7 @@ gog_chart_get_plot_area (GogChart *chart, GogViewAllocation *plot_area)
 }
 
 /**
- * gog_chart_set_plot_area :
+ * gog_chart_set_plot_area:
  * @chart : #GogChart
  * @plot_area  : #GogViewAllocation
  *
@@ -845,7 +852,19 @@ gog_chart_set_plot_area (GogChart *chart, GogViewAllocation const *plot_area)
 	gog_object_emit_changed (GOG_OBJECT (chart), TRUE);
 }
 
-/* FIXME: function description here */
+/**
+ * gog_chart_get_cardinality:
+ * @chart: a #GogChart
+ * @full: placeholder for full cardinality
+ * @visible: placeholder for visible cardinality
+ *
+ * Update and cache cardinality values if required, and returns
+ * full and visible cardinality. Full cardinality is the number of 
+ * chart elements that require a different style. Visible cardinality is
+ * the number of chart elements showed in chart legend.
+ *
+ * @full and @visible may be NULL.
+ **/ 
 void
 gog_chart_get_cardinality (GogChart *chart, unsigned *full, unsigned *visible)
 {
@@ -858,6 +877,7 @@ gog_chart_get_cardinality (GogChart *chart, unsigned *full, unsigned *visible)
 		chart->cardinality_valid = TRUE;
 		chart->full_cardinality = chart->visible_cardinality = 0;
 		for (ptr = chart->plots ; ptr != NULL ; ptr = ptr->next) {
+			gog_plot_update_cardinality (ptr->data, chart->visible_cardinality);
 			gog_plot_get_cardinality (ptr->data, &tmp_full, &tmp_visible);
 			chart->full_cardinality += tmp_full;
 			chart->visible_cardinality += tmp_visible;
@@ -1108,32 +1128,45 @@ static void
 grid_line_render (GSList *start_ptr, GogViewAllocation const *bbox) 
 {
 	GSList *ptr, *child_ptr;
+	GSList *minor_grid_lines = NULL;
+	GSList *major_grid_lines = NULL;
 	GogView *child_view, *axis_child_view;
 
-	/* Render minor lines first */
 	for (ptr = start_ptr; ptr != NULL; ptr = ptr->next) {
 		child_view = ptr->data;
 		if (IS_GOG_AXIS (child_view->model)) {
 			for (child_ptr = child_view->children; child_ptr != NULL; child_ptr = child_ptr->next) {
 				axis_child_view = child_ptr->data;
-				if (IS_GOG_GRID_LINE (axis_child_view->model) &&
-				    gog_grid_line_is_minor (GOG_GRID_LINE (axis_child_view->model)))
-					gog_view_render (axis_child_view, bbox);
-			}
-		}
-	}	    
-	/* then render major lines */
-	for (ptr = start_ptr; ptr != NULL; ptr = ptr->next) {
-		child_view = ptr->data;
-		if (IS_GOG_AXIS (child_view->model)) {
-			for (child_ptr = child_view->children; child_ptr != NULL; child_ptr = child_ptr->next) {
-				axis_child_view = child_ptr->data;
-				if (IS_GOG_GRID_LINE (axis_child_view->model) &&
-				    !gog_grid_line_is_minor (GOG_GRID_LINE (axis_child_view->model)))
-					gog_view_render (axis_child_view, bbox);
+				if (IS_GOG_GRID_LINE (axis_child_view->model)) {
+					if (gog_grid_line_is_minor (GOG_GRID_LINE (axis_child_view->model)))
+						minor_grid_lines = g_slist_prepend (minor_grid_lines, 
+										    axis_child_view);
+					else
+						major_grid_lines = g_slist_prepend (major_grid_lines, 
+										    axis_child_view);
+				}	
 			}
 		}
 	}	    
+	
+	/* Render stripes, minor first then major */
+	for (ptr = minor_grid_lines; ptr != NULL; ptr = ptr->next) {
+		gog_grid_line_view_render_stripes (ptr->data);
+	}
+	for (ptr = major_grid_lines; ptr != NULL; ptr = ptr->next) {
+		gog_grid_line_view_render_stripes (ptr->data);
+	}
+	
+	/* Render lines, minor first then major */
+	for (ptr = minor_grid_lines; ptr != NULL; ptr = ptr->next) {
+		gog_grid_line_view_render_lines (ptr->data);
+	}
+	for (ptr = major_grid_lines; ptr != NULL; ptr = ptr->next) {
+		gog_grid_line_view_render_lines (ptr->data);
+	}
+	
+	g_slist_free (minor_grid_lines);
+	g_slist_free (major_grid_lines);
 }
 
 static void
