@@ -34,7 +34,9 @@
 #include <gsf/gsf-input-textline.h>
 
 #include <string.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #include <errno.h>
 
 #define PREVIEW_HSIZE 150
@@ -42,20 +44,6 @@
 
 /* ------------------------------------------------------------------------- */
 
-#ifndef W_OK
-#define W_OK 2
-#endif
-
-#ifndef HAVE_G_ACCESS
-#ifdef G_OS_WIN32
-#error "A glib with g_access is required for Win32"
-#else
-#define g_access access
-#endif
-#endif
-
-/* ------------------------------------------------------------------------- */
-
 
 /**
  * go_gtk_button_new_with_stock
@@ -147,9 +135,9 @@ go_libglade_new (char const *gladefile, char const *root,
 }
 
 /**
- * go_editable_enters:
- * @window: dialog to affect.
- * @editable: Editable to affect.
+ * go_gtk_editable_enters:
+ * @window:
+ * @w: 
  *
  * Normally if there's an editable widget (such as #GtkEntry) in your
  * dialog, pressing Enter will activate the editable rather than the
@@ -158,7 +146,7 @@ go_libglade_new (char const *gladefile, char const *root,
  * function enables that behavior.
  **/
 void
-go_editable_enters (GtkWindow *window, GtkWidget *w)
+go_gtk_editable_enters (GtkWindow *window, GtkWidget *w)
 {
 	g_return_if_fail (GTK_IS_WINDOW (window));
 	g_signal_connect_swapped (G_OBJECT (w),
@@ -226,12 +214,11 @@ cb_parent_mapped (GtkWidget *parent, GtkWindow *window)
 
 /**
  * go_gtk_window_set_transient
- * @wbcg	: The calling window
+ * @toplevel	: The calling window
  * @window      : the transient window
  *
  * Make the window a child of the workbook in the command context, if there is
- * one.
- * The function duplicates the positioning functionality in
+ * one.  The function duplicates the positioning functionality in
  * gnome_dialog_set_parent, but does not require the transient window to be
  * a GnomeDialog.
  */
@@ -319,7 +306,7 @@ go_gtk_file_sel_dialog (GtkWindow *toplevel, GtkWidget *w)
 	delete_handler = g_signal_connect (w, "delete_event",
 		G_CALLBACK (gu_delete_handler), NULL);
 
-	gtk_widget_show_all (w);
+	gtk_widget_show (w);
 	gtk_grab_add (w);
 	gtk_main ();
 
@@ -511,7 +498,7 @@ go_gtk_select_image (GtkWindow *toplevel, const char *initial)
 }
 
 /**
- * gui_get_image_save_info:
+ * go_gui_get_image_save_info:
  * @toplevel: a #GtkWindow
  * @supported_formats: a #GSList of supported file formats
  * @ret_format: default file format
@@ -538,8 +525,19 @@ save_info_state_free (SaveInfoState *state)
 	g_free (state);
 }
 
+static void
+cb_format_combo_changed (GtkComboBox *combo, GtkWidget *expander)
+{
+	GOImageFormatInfo const *format_info;
+
+	format_info = go_image_get_format_info (gtk_combo_box_get_active (combo));
+	gtk_widget_set_sensitive (expander, 
+				  format_info != NULL && 
+				  format_info->is_dpi_useful);
+}
+
 char *
-gui_get_image_save_info (GtkWindow *toplevel, GSList *supported_formats,
+go_gui_get_image_save_info (GtkWindow *toplevel, GSList *supported_formats,
 			 GOImageFormat *ret_format, double *resolution)
 {
 	GOImageFormat format;
@@ -548,9 +546,10 @@ gui_get_image_save_info (GtkWindow *toplevel, GSList *supported_formats,
 	GtkFileChooser *fsel = gui_image_chooser_new (TRUE);
 	GtkWidget *expander = NULL;
 	GtkWidget *resolution_spin = NULL;
+	GtkWidget *resolution_table;
 	GladeXML *gui;
 	SaveInfoState *state;
-	const char *key = "gui_get_image_save_info";
+	const char *key = "go_gui_get_image_save_info";
 	char *uri = NULL;
 
 	state = g_object_get_data (G_OBJECT (toplevel), key);
@@ -582,7 +581,7 @@ gui_get_image_save_info (GtkWindow *toplevel, GSList *supported_formats,
 				format = GPOINTER_TO_UINT (l->data);
 				format_info = go_image_get_format_info (format);
 				gtk_combo_box_append_text (format_combo, _(format_info->desc)); 
-				if (format == state->format)
+				if (format == state->format) 
 					gtk_combo_box_set_active (format_combo, i);
 			}
 			if (gtk_combo_box_get_active (format_combo) < 0)
@@ -592,6 +591,11 @@ gui_get_image_save_info (GtkWindow *toplevel, GSList *supported_formats,
 			gtk_widget_hide (widget);
 		}
 
+		if (supported_formats != NULL && ret_format != NULL) {
+			widget = glade_xml_get_widget (gui, "image_save_dialog_extra");
+			gtk_file_chooser_set_extra_widget (fsel, widget);
+		}
+
 		/* Export setting expander */
 		expander = glade_xml_get_widget (gui, "export_expander");
 		if (resolution != NULL) {
@@ -602,8 +606,11 @@ gui_get_image_save_info (GtkWindow *toplevel, GSList *supported_formats,
 			gtk_widget_hide (expander);
 
 		if (resolution != NULL && supported_formats != NULL && ret_format != NULL) {
-			widget = glade_xml_get_widget (gui, "image_save_dialog_extra");
-			gtk_file_chooser_set_extra_widget (fsel, widget);
+			resolution_table = glade_xml_get_widget (gui, "resolution_table");
+		
+			cb_format_combo_changed (format_combo, resolution_table);	
+			g_signal_connect (GTK_WIDGET (format_combo), "changed", 
+					  G_CALLBACK (cb_format_combo_changed), resolution_table);
 		}
 
 		g_object_unref (G_OBJECT (gui));
@@ -794,6 +801,7 @@ go_gtk_help_button_init (GtkWidget *w, char const *data_dir, char const *app, ch
  * go_gtk_url_is_writeable:
  * @parent : #GtkWindow
  * @uri :
+ * @overwrite_by_default :
  *
  * Check if it makes sense to try saving.
  * If it's an existing file and writable for us, ask if we want to overwrite.
@@ -822,7 +830,7 @@ go_gtk_url_is_writeable (GtkWindow *parent, char const *uri,
 		go_gtk_notice_dialog (parent, GTK_MESSAGE_ERROR,
 				      _("%s\nis a directory name"), uri);
 		result = FALSE;
-	} else if (g_access (filename, W_OK) != 0 && errno != ENOENT) {
+	} else if (go_file_access (uri, W_OK) != 0 && errno != ENOENT) {
 		go_gtk_notice_dialog (parent, GTK_MESSAGE_ERROR,
 				      _("You do not have permission to save to\n%s"),
 				      uri);
@@ -972,6 +980,120 @@ go_pixbuf_new_from_file (char const *filename)
 	return pixbuf;
 }
 
+#ifndef HAVE_GDK_CAIRO_SET_SOURCE_PIXBUF 
+/* This is public from 2.8 onwards.   */
+static void
+gdk_cairo_set_source_pixbuf (cairo_t   *cr,
+			     GdkPixbuf *pixbuf,
+			     double     pixbuf_x,
+			     double     pixbuf_y)
+{
+  gint width = gdk_pixbuf_get_width (pixbuf);
+  gint height = gdk_pixbuf_get_height (pixbuf);
+  guchar *gdk_pixels = gdk_pixbuf_get_pixels (pixbuf);
+  int gdk_rowstride = gdk_pixbuf_get_rowstride (pixbuf);
+  int n_channels = gdk_pixbuf_get_n_channels (pixbuf);
+  guchar *cairo_pixels;
+  cairo_format_t format;
+  cairo_surface_t *surface;
+  static const cairo_user_data_key_t key;
+  int j;
+
+  if (n_channels == 3)
+    format = CAIRO_FORMAT_RGB24;
+  else
+    format = CAIRO_FORMAT_ARGB32;
+
+  cairo_pixels = g_malloc (4 * width * height);
+  surface = cairo_image_surface_create_for_data ((unsigned char *)cairo_pixels,
+						 format,
+						 width, height, 4 * width);
+  cairo_surface_set_user_data (surface, &key,
+			       cairo_pixels, (cairo_destroy_func_t)g_free);
+
+  for (j = height; j; j--)
+    {
+      guchar *p = gdk_pixels;
+      guchar *q = cairo_pixels;
+
+      if (n_channels == 3)
+	{
+	  guchar *end = p + 3 * width;
+	  
+	  while (p < end)
+	    {
+#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+	      q[0] = p[2];
+	      q[1] = p[1];
+	      q[2] = p[0];
+#else	  
+	      q[1] = p[0];
+	      q[2] = p[1];
+	      q[3] = p[2];
+#endif
+	      p += 3;
+	      q += 4;
+	    }
+	}
+      else
+	{
+	  guchar *end = p + 4 * width;
+	  guint t1,t2,t3;
+	    
+#define MULT(d,c,a,t) G_STMT_START { t = c * a + 0x7f; d = ((t >> 8) + t) >> 8; } G_STMT_END
+
+	  while (p < end)
+	    {
+#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+	      MULT(q[0], p[2], p[3], t1);
+	      MULT(q[1], p[1], p[3], t2);
+	      MULT(q[2], p[0], p[3], t3);
+	      q[3] = p[3];
+#else	  
+	      q[0] = p[3];
+	      MULT(q[1], p[0], p[3], t1);
+	      MULT(q[2], p[1], p[3], t2);
+	      MULT(q[3], p[2], p[3], t3);
+#endif
+	      
+	      p += 4;
+	      q += 4;
+	    }
+	  
+#undef MULT
+	}
+
+      gdk_pixels += gdk_rowstride;
+      cairo_pixels += 4 * width;
+    }
+
+  cairo_set_source_surface (cr, surface, pixbuf_x, pixbuf_y);
+  cairo_surface_destroy (surface);
+}
+#endif
+
+/**
+ * go_cairo_set_source_pixbuf:
+ * @cr: a #Cairo context
+ * @pixbuf: a #GdkPixbuf
+ * @pixbuf_x: X coordinate of location to place upper left corner of @pixbuf
+ * @pixbuf_y: Y coordinate of location to place upper left corner of @pixbuf
+ * 
+ * Sets the given pixbuf as the source pattern for the Cairo context.
+ * The pattern has an extend mode of %CAIRO_EXTEND_NONE and is aligned
+ * so that the origin of @pixbuf is @pixbuf_x, @pixbuf_y
+ *
+ **/
+void
+go_cairo_set_source_pixbuf (cairo_t   *cr,
+			    GdkPixbuf *pixbuf,
+			    double     pixbuf_x,
+			    double     pixbuf_y)
+{
+	gdk_cairo_set_source_pixbuf (cr, pixbuf, pixbuf_x, pixbuf_y);
+}
+
+
 #ifndef HAVE_GTK_DIALOG_GET_RESPONSE_FOR_WIDGET
 /* This is public from 2.8 onwards.   */
 static gint
