@@ -20,7 +20,7 @@
 /* ------------------------------------------------------------------------- */
 
 /* One less that the Julian day number of 19000101.  */
-static int date_origin = 0;
+static int date_origin_1900 = 0;
 /* Julian day number of 19040101.  */
 static int date_origin_1904 = 0;
 
@@ -28,19 +28,27 @@ static int date_origin_1904 = 0;
  * The serial number of 19000228.  Excel allocates a serial number for
  * the non-existing date 19000229.
  */
-static int const date_serial_19000228 = 59;
+static const int date_serial_19000228 = 59;
+
+/* 31-Dec-9999 */
+static const int date_serial_max_1900 = 2958465;
+static const int date_serial_max_1904 = 2957003;
+
 
 static void
 date_init (void)
 {
+	GDate date;
+
+	g_date_clear (&date, 1);
+
 	/* Day 1 means 1st of January of 1900 */
-	GDate* date = g_date_new_dmy (1, 1, 1900);
-	date_origin = g_date_get_julian (date) - 1;
+	g_date_set_dmy (&date, 1, 1, 1900);
+	date_origin_1900 = g_date_get_julian (&date) - 1;
 
 	/* Day 0 means 1st of January of 1904 */
-	g_date_set_dmy (date, 1, 1, 1904);
-	date_origin_1904 = g_date_get_julian (date);
-	g_date_free (date);
+	g_date_set_dmy (&date, 1, 1, 1904);
+	date_origin_1904 = g_date_get_julian (&date);
 }
 
 /* ------------------------------------------------------------------------- */
@@ -50,12 +58,12 @@ datetime_g_to_serial (GDate const *date, GODateConventions const *conv)
 {
 	int day;
 
-	if (!date_origin)
+	if (!date_origin_1900)
 		date_init ();
 
 	if (conv && conv->use_1904)
 		return g_date_get_julian (date) - date_origin_1904;
-	day = g_date_get_julian (date) - date_origin;
+	day = g_date_get_julian (date) - date_origin_1900;
 	return day + (day > date_serial_19000228);
 }
 
@@ -64,18 +72,22 @@ datetime_g_to_serial (GDate const *date, GODateConventions const *conv)
 void
 datetime_serial_to_g (GDate *res, int serial, GODateConventions const *conv)
 {
-	if (!date_origin)
+	if (!date_origin_1900)
 		date_init ();
 
 	g_date_clear (res, 1);
-	if (conv && conv->use_1904)
+	if (conv && conv->use_1904) {
+		if (serial > date_serial_max_1904)
+			return;
 		g_date_set_julian (res, serial + date_origin_1904);
-	else if (serial > date_serial_19000228) {
+	} else if (serial > date_serial_19000228) {
+		if (serial > date_serial_max_1900)
+			return;
 		if (serial == date_serial_19000228 + 1)
 			g_warning ("Request for date 19000229.");
-		g_date_set_julian (res, serial + date_origin - 1);
+		g_date_set_julian (res, serial + date_origin_1900 - 1);
 	} else
-		g_date_set_julian (res, serial + date_origin);
+		g_date_set_julian (res, serial + date_origin_1900);
 }
 
 /* ------------------------------------------------------------------------- */
@@ -88,7 +100,11 @@ datetime_timet_to_serial_raw (time_t t, GODateConventions const *conv)
 	GDate date;
 
         g_date_clear (&date, 1);
+#if GLIB_CHECK_VERSION(2,10,0)
+	g_date_set_time_t (&date, t);
+#else
 	g_date_set_time (&date, t);
+#endif
 	secs = tm->tm_hour * 3600 + tm->tm_min * 60 + tm->tm_sec;
 	return datetime_g_to_serial (&date, conv) +
 		secs / (double)SECS_PER_DAY;
@@ -147,17 +163,6 @@ datetime_timet_to_seconds (time_t t)
 
 /* ------------------------------------------------------------------------- */
 
-int
-datetime_g_days_between (GDate const* date1, GDate const *date2)
-{
-	g_return_val_if_fail (g_date_valid (date1), 0);
-	g_return_val_if_fail (g_date_valid (date2), 0);
-
-	return (int) (g_date_get_julian (date2) - g_date_get_julian (date1));
-}
-
-/* ------------------------------------------------------------------------- */
-
 int
 datetime_g_months_between (GDate const *date1, GDate const *date2)
 {
@@ -188,9 +193,9 @@ datetime_g_years_between (GDate const *date1, GDate const *date2)
 /* ------------------------------------------------------------------------- */
 
 /**
- * datetime_weeknum (GDate *date, int method)
- * @date      date
- * @method    week numbering method
+ * datetime_weeknum:
+ * @date:      date
+ * @method:    week numbering method
  *
  * Returns week number according to the given method.
  * 1:   Week starts on Sunday. Days before first Sunday are in week 0.
@@ -358,7 +363,7 @@ days_between_basis (GDate const *from, GDate const *to, basis_t basis)
 /* ------------------------------------------------------------------------- */
 
 /*
- * coup_cd
+ * go_coup_cd
  *
  * @res	      :
  * @settlement: GDate *
@@ -373,7 +378,7 @@ days_between_basis (GDate const *from, GDate const *to, basis_t basis)
  * this function does not depend on the basis of counting!
  */
 void
-coup_cd (GDate *result, GDate const *settlement, GDate const *maturity,
+go_coup_cd (GDate *result, GDate const *settlement, GDate const *maturity,
 	 int freq, gboolean eom, gboolean next)
 {
         int        months, periods;
@@ -421,7 +426,7 @@ coup_cd (GDate *result, GDate const *settlement, GDate const *maturity,
  * Currently, returns negative numbers if the branch is not implemented.
  */
 double
-coupdays (GDate const *settlement, GDate const *maturity,
+go_coupdays (GDate const *settlement, GDate const *maturity,
 	  GnmCouponConvention const *conv)
 {
 	GDate prev, next;
@@ -436,8 +441,8 @@ coupdays (GDate const *settlement, GDate const *maturity,
 		return 365.0 / conv->freq;
 	case BASIS_ACT_ACT:
 	default:
-		coup_cd (&next, settlement, maturity, conv->freq, conv->eom, TRUE);
-		coup_cd (&prev, settlement, maturity, conv->freq, conv->eom, FALSE);
+		go_coup_cd (&next, settlement, maturity, conv->freq, conv->eom, TRUE);
+		go_coup_cd (&prev, settlement, maturity, conv->freq, conv->eom, FALSE);
 		return days_between_basis (&prev, &next, BASIS_ACT_ACT);
         }
 }
@@ -450,16 +455,16 @@ coupdays (GDate const *settlement, GDate const *maturity,
  * the settlement date.
  */
 double
-coupdaybs (GDate const *settlement, GDate const *maturity,
+go_coupdaybs (GDate const *settlement, GDate const *maturity,
 	   GnmCouponConvention const *conv)
 {
 	GDate prev_coupon;
-	coup_cd (&prev_coupon, settlement, maturity, conv->freq, conv->eom, FALSE);
+	go_coup_cd (&prev_coupon, settlement, maturity, conv->freq, conv->eom, FALSE);
 	return days_between_basis (&prev_coupon, settlement, conv->basis);
 }
 
 /**
- * coupdaysnc :
+ * go_coupdaysnc :
  * @settlement :
  * @maturity :
  * @freq :
@@ -470,11 +475,11 @@ coupdaybs (GDate const *settlement, GDate const *maturity,
  * coupon date.
  **/
 double
-coupdaysnc (GDate const *settlement, GDate const *maturity,
+go_coupdaysnc (GDate const *settlement, GDate const *maturity,
 	    GnmCouponConvention const *conv)
 {
 	GDate next_coupon;
-	coup_cd (&next_coupon, settlement, maturity, conv->freq, conv->eom, TRUE);
+	go_coup_cd (&next_coupon, settlement, maturity, conv->freq, conv->eom, TRUE);
 	return days_between_basis (settlement, &next_coupon, conv->basis);
 }
 
@@ -484,3 +489,39 @@ gnm_date_convention_base (GODateConventions const *conv)
 	g_return_val_if_fail (conv != NULL, 1900);
 	return conv->use_1904 ? 1904 : 1900;
 }
+
+/* ------------------------------------------------------------------------- */
+
+char *
+go_date_weekday_name (GDateWeekday wd, gboolean abbrev)
+{
+	char buf[100];
+	GDate date;
+
+	g_return_val_if_fail (g_date_valid_weekday (wd), NULL);
+
+	g_date_clear (&date, 1);
+	g_date_set_dmy (&date, 5 + (int)wd, 3, 2006);
+	g_date_strftime (buf, sizeof (buf) - 1,
+			 abbrev ? "%a" : "%A",
+			 &date);
+	return g_strdup (buf);
+}
+
+char *
+go_date_month_name (GDateMonth m, gboolean abbrev)
+{
+	char buf[100];
+	GDate date;
+
+	g_return_val_if_fail (g_date_valid_month (m), NULL);
+
+	g_date_clear (&date, 1);
+	g_date_set_dmy (&date, 15, m, 2006);
+	g_date_strftime (buf, sizeof (buf) - 1,
+			 abbrev ? "%b" : "%B",
+			 &date);
+	return g_strdup (buf);
+}
+
+/* ------------------------------------------------------------------------- */
