@@ -22,7 +22,7 @@
 #include <goffice/goffice-config.h>
 #include "xl-surface.h"
 
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <gsf/gsf-impl-utils.h>
 
 #include <goffice/data/go-data-simple.h>
@@ -120,7 +120,30 @@ xl_contour_plot_update (GogObject *obj)
 	model->rows = 0;
 	model->columns = 0;
 
-	for (ptr = model->base.series ; ptr != NULL ; ptr = ptr->next) {
+	ptr = model->base.series;
+	series = ptr->data;
+	while (!gog_series_is_valid (GOG_SERIES (series))) {
+		ptr = ptr->next;
+		if (!ptr)
+			return;
+		series = ptr->data;
+	}
+	/* for first series, num_elements is used for zaxis, so we
+	can't use it to evaluate model->columns */
+	if (series->values[1].data != NULL) {
+		model->columns = go_data_vector_get_len (
+			GO_DATA_VECTOR (series->values[1].data));
+		if (series->values[0].data != NULL)
+				model->rows = go_data_vector_get_len (
+				GO_DATA_VECTOR (series->values[0].data));
+		if (model->rows < model->columns)
+			model->columns = model->rows;
+	} else if (series->values[0].data != NULL)
+		model->columns = go_data_vector_get_len (
+			GO_DATA_VECTOR (series->values[0].data));
+	model->rows = 1;
+
+	for (ptr = ptr->next; ptr != NULL; ptr = ptr->next) {
 		series = ptr->data;
 		if (!gog_series_is_valid (GOG_SERIES (series)))
 			continue;
@@ -153,8 +176,7 @@ get_y_vector (GogPlot *plot)
 	GSList *ptr;
 	int i;
 
-	if (contour->y_labels)
-		g_free (contour->y_labels);
+	g_free (contour->y_labels);
 	contour->y_labels = g_new0 (char const *, contour->base.rows);
 
 	for (ptr = plot->series, i = 0 ; ptr != NULL ; ptr = ptr->next, i++) {
@@ -195,14 +217,14 @@ xl_contour_plot_axis_get_bounds (GogPlot *plot, GogAxisType axis,
 	}
 	if (bounds->fmt == NULL && fmt != NULL)
 		bounds->fmt = go_format_ref (fmt);
-	bounds->val.minima = 0.;
-	bounds->logical.minima = 0.;
+	bounds->val.minima = 1.;
+	bounds->logical.minima = 1.;
 	bounds->logical.maxima = go_nan;
 	bounds->is_discrete    = TRUE;
 	bounds->center_on_ticks = TRUE;
 	bounds->val.maxima = (axis == GOG_AXIS_X)? 
-		contour->base.columns - 1.: 
-		contour->base.rows - 1.;
+		contour->base.columns: 
+		contour->base.rows;
 	return (GOData*) vec;
 }
 
@@ -210,8 +232,7 @@ static void
 xl_contour_plot_finalize (GObject *obj)
 {
 	XLContourPlot *plot = XL_CONTOUR_PLOT (obj);
-	if (plot->y_labels)
-		g_free (plot->y_labels);
+	g_free (plot->y_labels);
 	G_OBJECT_CLASS (xl_contour_parent_klass)->finalize (obj);
 }
 
